package Alien::Util;
use strict;
require Exporter;
use vars qw[ @ISA @EXPORT_OK ];
@ISA = 'Exporter';

use Cwd qw[cwd];
use IPC::Cmd        qw[run can_run];
use File::Fetch;
use File::Copy;
use File::Basename 	qw[dirname basename];
use Config qw'%Config';

@EXPORT_OK = qw[ modify copy_file process add_quotes find_progs fetch ];

=head1 NAME

Alien::Util - helper routines for Alien packages

=cut

=head2 C<< modify ONELINER FILES >>

Returns a reference to a list which will invoke
Perl (as C<$^X>) to perform a search-and-replace
on a set of files.

=cut

sub modify {
  my ($oneliner,@files) = @_;
  my $perl = Alien::Util::add_quotes($^X);

  return [$perl, '-pi.bak', '-we', $oneliner, add_quotes(@files) ]
};

=head2 C<< copy_file SRC DEST >>

Returns a code reference which will copy the file
given as C<SRC> to the name C<DEST>

=cut

sub copy_file {
  my @args = @_;
  return sub {
    print "Copying $args[0] to $args[1]\n";
    if (! $::DRY_RUN) {
      copy(@args)
        or die "Couldn't copy $args[0] to $args[1]: $!";
    };
  };
};

=head2 C<< process COMMANDS >>

Will process all the given commands. If the
item is an array reference, it will be
passed to L<IPC::Run>. If the item is a code
reference, it will be called.

The global C<$::DRY_RUN> variable will make all commands
just print out their intentions without
actually performing the actions.

=cut

sub process {
  my (@cmds) = @_;
  my $dry_run = $::DRY_RUN;
  my $verbose = $::VERBOSE;
  for my $cmd (@cmds) {
    if (ref $cmd eq 'CODE') {
      $cmd->($dry_run)
    } else {
      next unless @$cmd;
      if ($dry_run || $verbose) {
        local $" = " ";
        print "[@$cmd]\n";
      } else {
        unless( scalar run( command => [@$cmd], verbose => 1, )) {
          die qq[Could not run '@$cmd': $!]
        }
      };
    }
  }
};

=head2 C<< add_quotes ITEMS >>

Add (double) quotes to a string if it needs
them. This is mostly useful when passing items
to an interpolated Win32 command line.

Quoting is not always necessary for Win32 commands
if they are invoked via C<< system LIST >>, as it
is the One True Way of command invocation.

This function returns a list or the first item of
the list, depending on the context.

=cut

sub add_quotes {
  my @res = map {
     (/ / and ! /"/) ? qq{"$_"} : $_ ;
  } @_;
  wantarray ? @res : $res[0]
};

=head2 C<< find_progs PROGS >>

Checks that the list of programs required
for building the external library can be
found in the path. The list is currently
given as a list of names.

The routine dies if at least one of the prerequisite
programs cannot be found.

The programs are first checked via L<Config>,
so that C<make> automatically becomes its
platform local equivalent.

It returns a hash reference with the full
path names to the programs.

=cut

sub find_progs {
    my $progs = {};
    my @progs = @_;
    my @missing;
    ### find required programs ###
    for my $pgm ( @progs ) {
        $progs->{$pgm} = can_run($Config{$pgm} || $pgm)
          or push @missing, qq[Can not locate required program '$pgm'];
    };
    if (@missing) {
      die join "", @missing;
    };
    return $progs;
};

=head2 C<< decompress PROGRAMS, ARCHIVE >>

Calls C<gzip> and C<tar> on the archive
file to extract it.

=cut

sub decompress {
    my ($progs, $xdir, $archive) = @_;

    -f $archive
      or die "Internal error: '$archive' should be a file, but does not exist.";

    my $dir = dirname($archive);
    chdir $dir
      or die "Couldn't chdir() to '$dir': $!";

    (my $tar = $archive) =~ s!\.gz$!!;
    (my $tardir = $tar) =~ s!\.tar$!!;

    my @cmds;
    if ($^O eq 'MSWin32') {
       # Decompress in two steps, because it's safer, at least
       # on Win32 with unxutils:

       #warn $archive;
       #warn $tar;

      if (! -d $tardir) {
        unshift @cmds, [$progs->{tar},'-xf', $tar];
        if (! -f $tar) {
            unshift @cmds, [$progs->{gzip},'-d', $archive]
        } else {
            print "'$tar' already exists, not decompressing again.\n";
        };
      } else {
        print "'$tardir' already exists, not decompressing again.\n";
      };
    } else {
      ($archive, $tar) = Alien::Util::add_quotes($archive, $tar);
      if (! -d $tardir) {
          push @cmds, qq[$progs->{gzip} -cd $archive | $progs->{tar} -xf -];
      } else {
        print "'$tardir' already exists, not decompressing again.\n";
      };
    };

    process( @cmds );

    print "succesfully extracted library\n";

    return '';
};

=head2 C<< fetch URLS >>

Tries to fetch the given URLs using L<File::Fetch>.

Dies if none of them can be fetched.

Returns the URL sucessfully fetched.

=cut

sub fetch {
    my @urls = @_;

    my $cwd = cwd;
    my $dir  = File::Spec->catdir($cwd, 'src');

    for my $url (@urls) {
        (my $guessed_name) = ($url =~ m!/([^/]+)$!g);
        #warn $guessed_name;
        if (-f File::Spec->catfile($dir,$guessed_name)) {
            print "$url seems to exist as $guessed_name already, not fetching\n";
            return File::Spec->catfile($dir,$guessed_name)
        } else {
            print "Trying to fetch $url\n";

            my $ff  = File::Fetch->new( uri => $url ) or next;
            my $loc = $ff->fetch( to => $dir );

            next unless $loc;

            #use Data::Dumper;
            #print Dumper $loc;

            return $loc;
        };
    }

    ### if we got here, we weren't able to fetch the source files ###
    die "Unable to fetch the library sources -- can not continue, sorry\n";
}



1;