package Alien::Iconv;
use strict;
use vars qw($VERSION $LIB_VERSION @ARCHIVES @URLS);

use Config qw'%Config';

$VERSION = '0.01';
$LIB_VERSION = '1.9.2';
@ARCHIVES = ("libiconv-$LIB_VERSION.tar.gz");
@URLS = ("http://ftp.gnu.org/pub/gnu/libiconv/$ARCHIVES[0]");


=head1 NAME

Alien::Iconv - install a local iconv just for Perl

=head1 DESCRIPTION

This module allows you to install a local instance of the C<libiconv>
library via L<CPAN> (or L<CPANPLUS>).

It does not yet try to find an already existing instance but will
always install a new library. If this is not what you want, submit
patches.

=head1 INSTALLATION

This package downloads the full C<libiconv> sources as found
at L<http://ftp.gnu.org/pub/gnu/libiconv/> at installation
time.

If you don't want the download, create a directory C<src> and put the
compressed file as C<src/libiconv-1.9.2.tar.gz> in the build directory.

=head1 METHODS

This package exports the following methods to query
the installation directories:

=head2 C<prefix();>

Returns the prefix (ie, the directory) where the C<include> and C<lib>
directores will live.
On a standard unix system, this might be C</usr/local/>

For this module, it will be C<$INC{'Alien/Iconv'}>.

=cut

my $prefix;
sub prefix {
    return $prefix if $prefix;

    my $ext = '.pm';
    $prefix = join '/', split '::', __PACKAGE__;
    $prefix = File::Spec->rel2abs( $INC{$prefix.$ext} );
    $prefix =~ s/$ext$//;

    return $prefix;
}

=head2 lib();

Returns the full path to the C<lib> directory for the C<libxml2>
libraries.
On standard unix systems, this might be C</usr/local/lib>

For this module, it will be C<$INC{'Alien/Iconv'}/lib>

=head2 include()

Returns the full path to the C<include> directory for the C<libpng>
libraries.
On standard unix systems, this might be C</usr/local/include>

For this module, it will be C<$INC{'Alien/Iconv'}/include>

=cut

for my $sub (qw[lib include]) {
    no strict 'refs';
    *$sub = sub {
                my $self = shift;
                return File::Spec->catdir( $self->prefix, $sub );
            };
}

=head2 C<< version() >>

Returns the version of the zlib libraries installed by this version
of C<Alien::XML::LibXML>

=cut

sub version { return $LIB_VERSION; }

=head2 C<< lib_files() >>

Returns a list of full paths to the files that can be found in the
C<< Alien::XML::LibXML->lib >> directory. This corresponds to all created
libraries.

=head2 C<< include_files() >>

Returns a list of full paths to the files that can be found in the
C<< Alien::XML::LibXML->include >> direcotry. This corresponds to all copied
header files.

=cut

# Need to add all other prerequisites here as well
my $files = {
    include => [qw[iconv.h libcharset.h localcharset.h]],
    lib     => [qw[charset.lib iconv.lib]],
};

for my $acc (qw[include_files lib_files]) {
    no strict 'refs';
    *$acc = sub {
                my $self    = shift;
                my ($type)  = $acc =~ /(\w+)_files/;

                return map {
                            File::Spec->catfile( $self->$type(), $_)
                        } @{$files->{$type}};
    }
};

=head2 C<< archives() >>

Returns the archive filenames. In this case it's C<...>

=head2 C<< lib_version() >>

Returns the version of the linked library.

=head2 C<< urls() >>

Returns the urls to fetch the source code from.

=cut

for my $name qw(archives urls) {
  no strict 'refs';
  *$name = sub() { return @{uc $name} };
};

sub lib_version { return $LIB_VERSION };

=head1 TODO

=over 4

=item *

Probe for system installed version of C<libxml2>
instead of always downloading and building.

=head1 LICENSE

The Perl code in this distribution can be distributed under
the same terms as Perl itself. The C<libxml2> code is copyright (c)
2001 by the JFIF and has the following copyright note:

=head1 AUTHOR

The author of the Perl code in this package is Max Maischein
L<corion@cpan.org>. Much of the code was cannibalized from
L<Alien::Zlib>.
The authors of the C<libxml2> libraries
are noted in the C<README> text file of the C<libxml2>
distribution.

=cut

1;